@@ -1,9 +1,8 @@
 /*
  * bmon.h              All-include Header
  *
- * $Id: bmon.h 1 2004-10-17 17:32:34Z tgr $
- *
- * Copyright (c) 2001-2004 Thomas Graf <tgraf@suug.ch>
+ * Copyright (c) 2001-2013 Thomas Graf <tgraf@suug.ch>
+ * Copyright (c) 2013 Red Hat, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -28,30 +27,56 @@
 #define __BMON_BMON_H_
 
 #include <bmon/config.h>
+#include <bmon/list.h>
 
-typedef uint64_t b_cnt_t;
-typedef uint32_t rate_cnt_t;
+extern int start_time;
 
 typedef struct timestamp_s
 {
-	int64_t tv_sec;
-	int64_t tv_usec;
+	int64_t		tv_sec;
+	int64_t		tv_usec;
 } timestamp_t;
 
+typedef struct xdate_s
+{
+	struct tm	d_tm;
+	unsigned int	d_usec;
+} xdate_t;
+
 enum {
 	EMPTY_LIST = 1,
-	END_OF_LIST = 2,
+	END_OF_LIST = 2
 };
 
-#define BUG()			                 \
-	do {		                         \
-		fprintf(stderr, "BUG: %s:%d\n", __FILE__, __LINE__); \
-		exit(EINVAL);	                 \
+#define BUG()								\
+	do {								\
+		fprintf(stderr, "BUG: %s:%d\n", __FILE__, __LINE__);	\
+                assert(0); \
+		exit(EINVAL);						\
 	} while (0);
 
 #define ARRAY_SIZE(X) (sizeof(X) / sizeof((X)[0]))
 
+#if defined __GNUC__
 #define __init __attribute__ ((constructor))
 #define __exit __attribute__ ((destructor))
+#else
+#define __init
+#define __exit
+#endif
+
+#ifdef DEBUG
+#define DBG(FMT, ARG...)					\
+	do {							\
+		fprintf(stderr,					\
+			"[DBG] %20s:%-4u %s: " FMT "\n",	\
+			__FILE__, __LINE__,			\
+			__PRETTY_FUNCTION__, ##ARG);		\
+	} while (0)
+#else
+#define DBG(FMT, ARG...)					\
+	do {							\
+	} while (0)
+#endif
 
 #endif
