@@ -1,9 +1,8 @@
 /*
  * config.h             Global Config
  *
- * $Id: config.h 1 2004-10-17 17:32:34Z tgr $
- *
- * Copyright (c) 2001-2004 Thomas Graf <tgraf@suug.ch>
+ * Copyright (c) 2001-2013 Thomas Graf <tgraf@suug.ch>
+ * Copyright (c) 2013 Red Hat, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -41,11 +40,31 @@
 #include <limits.h>
 #include <stdlib.h>
 #include <math.h>
+#include <inttypes.h>
+#include <sys/file.h>
+#include <assert.h>
+#include <syslog.h>
+#include <sys/wait.h>
+#include <dirent.h>
+#include <values.h>
 
-#include <sys/time.h>
+#if TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# if HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
 
-#ifdef TIME_WITH_SYS_TIME
-#include <time.h>
+#ifdef HAVE_VFORK_H
+#include <vfork.h>
+#endif
+
+#if !HAVE_WORKING_VFORK
+#define vfork fork
 #endif
 
 #if defined HAVE_STRING_H
@@ -82,31 +101,26 @@
 
 #include <sys/stat.h>
 #include <grp.h>
+#include <pwd.h>
 #include <errno.h>
 
 #if defined HAVE_SYS_UTSNAME_H
 #include <sys/utsname.h>
 #endif
 
-#if defined HAVE_NCURSES
-#  if defined HAVE_NCURSES_NCURSES_H
-#    include <ncurses/ncurses.h>
-#  elif defined HAVE_NCURSES_H
-#    include <ncurses.h>
-#  elif defined HAVE_CURSES_H
-#    warn "*** WARNING: Broken system: libncurses is present but not ncurses.h, using curses.h ***"
-#    include <curses.h>
-#  else
-#    error "*** ERROR: Broken system: libncurses present but no corresponding header file ***"
-#  endif /* HAVE_*_*CURSES_H */
-#  if !defined HAVE_CURSES
-#    define HAVE_CURSES
-#  endif /* !HAVE_CURSES */
-#elif defined HAVE_CURSES
+#if defined HAVE_NCURSESW_CURSES_H
+#  include <ncursesw/curses.h>
+#elif defined HAVE_NCURSESW_H
+#  include <ncursesw.h>
+#elif defined HAVE_NCURSES_CURSES_H
+#  include <ncurses/curses.h>
+#elif defined HAVE_NCURSES_H
+#  include <ncurses.h>
+#elif defined HAVE_CURSES_H
 #  include <curses.h>
 #else
-#  error "*** ERROR: Neither ncurses nor curses is present on your system. ***"
-#endif /* HAVE_[N]CURSES */
+#  error "SysV or X/Open-compatible Curses header file required"
+#endif
 
 #include <netinet/in.h>
 
@@ -117,10 +131,28 @@
 #include <sys/socket.h>
 #include <arpa/inet.h>
 
+#include <confuse.h>
 
 #ifndef IFNAMSIZ
 #define IFNAMSIZ 16
 #endif
 
+#ifndef SCNu64
+#define SCNu64 "llu"
+#endif
+
+#ifndef PRIu64
+#define PRIu64 "llu"
+#endif
+
+#ifndef PRId64
+#define PRId64 "lld"
+#endif
+
+#ifndef PRIX64
+#define PRIX64 "X"
+#endif
+
+#define DEFAULT_GROUP "intf"
 
 #endif
