@@ -1,7 +1,8 @@
 /*
  * input.h             Input API
  *
- * Copyright (c) 2001-2004 Thomas Graf <tgraf@suug.ch>
+ * Copyright (c) 2001-2013 Thomas Graf <tgraf@suug.ch>
+ * Copyright (c) 2013 Red Hat, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,11 +28,16 @@
 
 #include <bmon/bmon.h>
 #include <bmon/conf.h>
+#include <bmon/module.h>
+
+extern int input_set(const char *);
+extern void input_register(struct bmon_module *);
+extern void input_read(void);
 
 struct reader_timing
 {
-	timestamp_t rt_last_read;
-	timestamp_t rt_next_read;
+	timestamp_t rt_last_read;	/* timestamp taken before read */
+	timestamp_t rt_next_read;	/* estimated next read */
 
 	struct {
 		float v_error;
@@ -43,26 +49,4 @@ struct reader_timing
 
 extern struct reader_timing rtiming;
 
-struct input_module
-{
-	char *                im_name;
-	void                (*im_read)(void);
-	void                (*im_set_opts)(tv_t *);
-	int                 (*im_probe)(void);
-	void                (*im_init)(void);
-	void                (*im_shutdown)(void);
-	int                   im_no_default;
-	int                   im_enable;
-	struct input_module * im_next;
-};
-
-extern void set_input(const char *);
-extern void set_sec_input(const char *);
-extern void register_input_module(struct input_module *);
-extern void register_secondary_input_module(struct input_module *);
-extern void input_init(void);
-extern void input_shutdown(void);
-extern void input_read(void);
-extern const char * get_preferred_input_name(void);
-
 #endif
